@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -31,8 +31,17 @@
 #include <algorithm>
 #include <iostream>
 #include <cstdio>
+#include <cstddef>
+#include <cmath>
 #include <sstream>
 
+// VS2013 doesn't support alignof
+#if defined(_MSC_VER) && _MSC_VER <= 1800
+#define LOVE_ALIGNOF(x) __alignof(x)
+#else
+#define LOVE_ALIGNOF(x) alignof(x)
+#endif
+
 namespace love
 {
 
@@ -84,6 +93,38 @@ static int w__eq(lua_State *L)
 	return 1;
 }
 
+// For use with the love object pointer -> Proxy pointer registry.
+// Using the pointer directly via lightuserdata would be ideal, but LuaJIT
+// cannot use lightuserdata with more than 47 bits whereas some newer arm64
+// architectures allow pointers which use more than that.
+static lua_Number luax_computeloveobjectkey(lua_State *L, love::Object *object)
+{
+	// love objects should be allocated on the heap, and thus are subject
+	// to the alignment rules of operator new / malloc. Lua numbers (doubles)
+	// can store all possible integers up to 2^53. We can store pointers that
+	// use more than 53 bits if their alignment is guaranteed to be more than 1.
+	// For example an alignment requirement of 8 means we can shift the
+	// pointer's bits by 3.
+	const size_t minalign = LOVE_ALIGNOF(std::max_align_t);
+	uintptr_t key = (uintptr_t) object;
+
+	if ((key & (minalign - 1)) != 0)
+	{
+		luaL_error(L, "Cannot push love object to Lua: unexpected alignment "
+				   "(pointer is %p but alignment should be %d)", object, minalign);
+	}
+
+	static const size_t shift = (size_t) log2(LOVE_ALIGNOF(std::max_align_t));
+
+	key >>= shift;
+
+	// Make sure our key isn't larger than 2^53.
+	if (key > 0x20000000000000ULL)
+		luaL_error(L, "Cannot push love object to Lua: pointer value %p is too large", object);
+
+	return (lua_Number) key;
+}
+
 static int w__release(lua_State *L)
 {
 	Proxy *p = (Proxy *) lua_touserdata(L, 1);
@@ -100,7 +141,8 @@ static int w__release(lua_State *L)
 		if (lua_istable(L, -1))
 		{
 			// loveobjects[object] = nil
-			lua_pushlightuserdata(L, object);
+			lua_Number objectkey = luax_computeloveobjectkey(L, object);
+			lua_pushnumber(L, objectkey);
 			lua_pushnil(L);
 			lua_settable(L, -3);
 		}
@@ -209,6 +251,13 @@ void luax_pushstring(lua_State *L, const std::string &str)
 	lua_pushlstring(L, str.data(), str.size());
 }
 
+void luax_pushpointerasstring(lua_State *L, const void *pointer)
+{
+	char str[sizeof(void *)];
+	memcpy(str, &pointer, sizeof(void *));
+	lua_pushlstring(L, str, sizeof(void *));
+}
+
 bool luax_boolflag(lua_State *L, int table_index, const char *key, bool defaultValue)
 {
 	lua_getfield(L, table_index, key);
@@ -241,7 +290,7 @@ double luax_numberflag(lua_State *L, int table_index, const char *key, double de
 {
 	lua_getfield(L, table_index, key);
 
-	int retval;
+	double retval;
 	if (!lua_isnumber(L, -1))
 		retval = defaultValue;
 	else
@@ -452,7 +501,7 @@ int luax_register_type(lua_State *L, love::Type *type, ...)
 	return 0;
 }
 
-void luax_gettypemetatable(lua_State *L, love::Type &type)
+void luax_gettypemetatable(lua_State *L, const love::Type &type)
 {
 	const char *name = type.getName();
 	lua_getfield(L, LUA_REGISTRYINDEX, name);
@@ -550,14 +599,16 @@ void luax_pushtype(lua_State *L, love::Type &type, love::Object *object)
 	luax_getregistry(L, REGISTRY_OBJECTS);
 
 	// The table might not exist - it should be insisted in luax_register_type.
-	if (!lua_istable(L, -1))
+	if (lua_isnoneornil(L, -1))
 	{
 		lua_pop(L, 1);
 		return luax_rawnewtype(L, type, object);
 	}
 
+	lua_Number objectkey = luax_computeloveobjectkey(L, object);
+
 	// Get the value of loveobjects[object] on the stack.
-	lua_pushlightuserdata(L, object);
+	lua_pushnumber(L, objectkey);
 	lua_gettable(L, -2);
 
 	// If the Proxy userdata isn't in the instantiated types table yet, add it.
@@ -567,7 +618,7 @@ void luax_pushtype(lua_State *L, love::Type &type, love::Object *object)
 
 		luax_rawnewtype(L, type, object);
 
-		lua_pushlightuserdata(L, object);
+		lua_pushnumber(L, objectkey);
 		lua_pushvalue(L, -2);
 
 		// loveobjects[object] = Proxy.
@@ -623,7 +674,7 @@ int luax_convobj(lua_State *L, int idx, const char *mod, const char *fn)
 	return 0;
 }
 
-int luax_convobj(lua_State *L, int idxs[], int n, const char *mod, const char *fn)
+int luax_convobj(lua_State *L, const int idxs[], int n, const char *mod, const char *fn)
 {
 	luax_getfunction(L, mod, fn);
 	for (int i = 0; i < n; i++)
@@ -638,6 +689,12 @@ int luax_convobj(lua_State *L, int idxs[], int n, const char *mod, const char *f
 	return 0;
 }
 
+int luax_convobj(lua_State *L, const std::vector<int>& idxs, const char *module, const char *function)
+{
+	const int *idxPtr = idxs.size() > 0 ? &idxs[0] : nullptr;
+	return luax_convobj(L, idxPtr, (int) idxs.size(), module, function);
+}
+
 int luax_pconvobj(lua_State *L, int idx, const char *mod, const char *fn)
 {
 	// Convert string to a file.
@@ -649,7 +706,7 @@ int luax_pconvobj(lua_State *L, int idx, const char *mod, const char *fn)
 	return ret;
 }
 
-int luax_pconvobj(lua_State *L, int idxs[], int n, const char *mod, const char *fn)
+int luax_pconvobj(lua_State *L, const int idxs[], int n, const char *mod, const char *fn)
 {
 	luax_getfunction(L, mod, fn);
 	for (int i = 0; i < n; i++)
@@ -662,6 +719,12 @@ int luax_pconvobj(lua_State *L, int idxs[], int n, const char *mod, const char *
 	return ret;
 }
 
+int luax_pconvobj(lua_State *L, const std::vector<int>& idxs, const char *module, const char *function)
+{
+	const int *idxPtr = idxs.size() > 0 ? &idxs[0] : nullptr;
+	return luax_pconvobj(L, idxPtr, (int) idxs.size(), module, function);
+}
+
 int luax_insist(lua_State *L, int idx, const char *k)
 {
 	// Convert to absolute index if necessary.
@@ -874,9 +937,39 @@ void luax_register(lua_State *L, const char *name, const luaL_Reg *l)
 	}
 }
 
+void luax_runwrapper(lua_State *L, const char *filedata, size_t datalen, const char *filename, const love::Type &type, void *ffifuncs)
+{
+	luax_gettypemetatable(L, type);
+
+	// Load and execute ImageData.lua, sending the metatable and the ffi
+	// functions struct pointer as arguments.
+	if (lua_istable(L, -1))
+	{
+		luaL_loadbuffer(L, filedata, datalen, filename);
+		lua_pushvalue(L, -2);
+		if (ffifuncs != nullptr)
+			luax_pushpointerasstring(L, ffifuncs);
+		else
+			lua_pushnil(L);
+		lua_call(L, 2, 0);
+	}
+
+	// Pop the metatable.
+	lua_pop(L, 1);
+}
+
 Type *luax_type(lua_State *L, int idx)
 {
 	return Type::byName(luaL_checkstring(L, idx));
 }
 
+int luax_resume(lua_State *L, int nargs)
+{
+#if LUA_VERSION_NUM >= 502
+	return lua_resume(L, nullptr, nargs);
+#else
+	return lua_resume(L, nargs);
+#endif
+}
+
 } // love
