@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -101,6 +101,15 @@ Audio::Audio()
 	love::thread::disableSignals();
 #endif
 
+	// Before opening new device, check if recording
+	// is requested.
+	if (getRequestRecordingPermission())
+	{
+		if (!hasRecordingPermission())
+			// Request recording permission on some OSes.
+			requestRecordingPermission();
+	}
+
 	// Passing null for default device.
 	device = alcOpenDevice(nullptr);
 
@@ -402,6 +411,15 @@ const std::vector<love::audio::RecordingDevice*> &Audio::getRecordingDevices()
 	std::vector<std::string> devnames;
 	std::vector<love::audio::RecordingDevice*> devices;
 
+	// If recording permission is not granted, inform user about it
+	// and return empty list.
+	if (!hasRecordingPermission() && getRequestRecordingPermission())
+	{
+		showRecordingPermissionMissingDialog();
+		capture.clear();
+		return capture;
+	}
+
 	std::string defaultname(alcGetString(NULL, ALC_CAPTURE_DEFAULT_DEVICE_SPECIFIER));
 
 	//no device name obtained from AL, fallback to reading from device
