@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2014 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -30,42 +30,47 @@
 #include "joystick/Joystick.h"
 #include "thread/threads.h"
 
-// STL
+// C++
 #include <queue>
+#include <vector>
 
 namespace love
 {
 namespace event
 {
+
 class Message : public Object
 {
-private:
-	std::string name;
-	Variant *args[4];
-	int nargs;
-
 public:
-	Message(const std::string &name, Variant *a = NULL, Variant *b = NULL, Variant *c = NULL, Variant *d = NULL);
+
+	Message(const std::string &name, const std::vector<Variant> &vargs = {});
 	~Message();
 
 	int toLua(lua_State *L);
 	static Message *fromLua(lua_State *L, int n);
-};
+
+	const std::string name;
+	const std::vector<Variant> args;
+
+}; // Message
 
 class Event : public Module
 {
 public:
-	Event();
 	virtual ~Event();
 
+	// Implements Module.
+	virtual ModuleType getModuleType() const { return M_EVENT; }
+
 	void push(Message *msg);
 	bool poll(Message *&msg);
 	virtual void clear();
 
 	virtual void pump() = 0;
+	virtual Message *wait() = 0;
 
 protected:
-	thread::Mutex *mutex;
+	love::thread::MutexRef mutex;
 	std::queue<Message *> queue;
 
 }; // Event
