@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -138,6 +138,7 @@ int w_mount(lua_State *L)
 		bool append = luax_optboolean(L, startidx + 1, false);
 
 		luax_pushboolean(L, instance()->mount(data, archive.c_str(), mountpoint, append));
+		return 1;
 	}
 	else if (luax_istype(L, 1, DroppedFile::type))
 	{
@@ -213,7 +214,10 @@ File *luax_getfile(lua_State *L, int idx)
 		file = instance()->newFile(filename);
 	}
 	else
+	{
 		file = luax_checkfile(L, idx);
+		file->retain();
+	}
 
 	return file;
 }
@@ -226,7 +230,6 @@ FileData *luax_getfiledata(lua_State *L, int idx)
 	if (lua_isstring(L, idx) || luax_istype(L, idx, File::type))
 	{
 		file = luax_getfile(L, idx);
-		file->retain();
 	}
 	else if (luax_istype(L, idx, FileData::type))
 	{
@@ -259,7 +262,6 @@ Data *luax_getdata(lua_State *L, int idx)
 	if (lua_isstring(L, idx) || luax_istype(L, idx, File::type))
 	{
 		file = luax_getfile(L, idx);
-		file->retain();
 	}
 	else if (luax_istype(L, idx, Data::type))
 	{
@@ -488,24 +490,17 @@ int w_read(lua_State *L)
 	if (data == nullptr)
 		return luax_ioError(L, "File could not be read.");
 
-	int nret = 0;
-
 	if (ctype == love::data::CONTAINER_DATA)
-	{
 		luax_pushtype(L, data);
-		nret = 1;
-	}
 	else
-	{
 		lua_pushlstring(L, (const char *) data->getData(), data->getSize());
-		lua_pushinteger(L, data->getSize());
-		nret = 2;
-	}
+
+	lua_pushinteger(L, data->getSize());
 
 	// Lua has a copy now, so we can free it.
 	data->release();
 
-	return nret;
+	return 2;
 }
 
 static int w_write_or_append(lua_State *L, File::Mode mode)
@@ -727,7 +722,7 @@ static void replaceAll(std::string &str, const std::string &substr, const std::s
 
 int loader(lua_State *L)
 {
-	std::string modulename = luax_tostring(L, 1);
+	std::string modulename = luax_checkstring(L, 1);
 
 	for (char &c : modulename)
 	{
@@ -768,7 +763,7 @@ static const char *library_extensions[] =
 
 int extloader(lua_State *L)
 {
-	const char *filename = lua_tostring(L, -1);
+	std::string filename = luax_checkstring(L, 1);
 	std::string tokenized_name(filename);
 	std::string tokenized_function(filename);
 
