@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -38,7 +38,7 @@ class FenceSync
 {
 public:
 
-	FenceSync() {}
+	FenceSync() : sync(0) {}
 	~FenceSync();
 
 	bool fence();
