@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -59,6 +59,7 @@ Graphics::Graphics()
 	, mainVAO(0)
 {
 	gl = OpenGL();
+	Canvas::resetFormatSupport();
 
 	auto window = getInstance<love::window::Window>(M_WINDOW);
 
@@ -342,7 +343,7 @@ void Graphics::draw(const DrawIndexedCommand &cmd)
 	++drawCalls;
 }
 
-static inline void advanceVertexOffsets(const vertex::Attributes &attributes, vertex::Buffers &buffers, int vertexcount)
+static inline void advanceVertexOffsets(const vertex::Attributes &attributes, vertex::BufferBindings &buffers, int vertexcount)
 {
 	// TODO: Figure out a better way to avoid touching the same buffer multiple
 	// times, if multiple attributes share the buffer.
@@ -355,16 +356,17 @@ static inline void advanceVertexOffsets(const vertex::Attributes &attributes, ve
 
 		auto &attrib = attributes.attribs[i];
 
-		uint32 bufferbit = 1u << attrib.bufferindex;
+		uint32 bufferbit = 1u << attrib.bufferIndex;
 		if ((touchedbuffers & bufferbit) == 0)
 		{
 			touchedbuffers |= bufferbit;
-			buffers.info[attrib.bufferindex].offset += attrib.stride * vertexcount;
+			const auto &layout = attributes.bufferLayouts[attrib.bufferIndex];
+			buffers.info[attrib.bufferIndex].offset += layout.stride * vertexcount;
 		}
 	}
 }
 
-void Graphics::drawQuads(int start, int count, const vertex::Attributes &attributes, const vertex::Buffers &buffers, love::graphics::Texture *texture)
+void Graphics::drawQuads(int start, int count, const vertex::Attributes &attributes, const vertex::BufferBindings &buffers, love::graphics::Texture *texture)
 {
 	const int MAX_VERTICES_PER_DRAW = LOVE_UINT16_MAX;
 	const int MAX_QUADS_PER_DRAW    = MAX_VERTICES_PER_DRAW / 4;
@@ -393,7 +395,7 @@ void Graphics::drawQuads(int start, int count, const vertex::Attributes &attribu
 	}
 	else
 	{
-		vertex::Buffers bufferscopy = buffers;
+		vertex::BufferBindings bufferscopy = buffers;
 		if (start > 0)
 			advanceVertexOffsets(attributes, bufferscopy, start * 4);
 
@@ -820,6 +822,7 @@ void Graphics::bindCachedFBO(const RenderTargets &targets)
 	else
 	{
 		int msaa = targets.getFirstTarget().canvas->getMSAA();
+		bool hasDS = targets.depthStencil.canvas != nullptr;
 
 		glGenFramebuffers(1, &fbo);
 		gl.bindFramebuffer(OpenGL::FRAMEBUFFER_ALL, fbo);
@@ -864,11 +867,21 @@ void Graphics::bindCachedFBO(const RenderTargets &targets)
 		for (const auto &rt : targets.colors)
 			attachCanvas(rt);
 
-		if (targets.depthStencil.canvas != nullptr)
+		if (hasDS)
 			attachCanvas(targets.depthStencil);
 
 		if (ncolortargets > 1)
 			glDrawBuffers(ncolortargets, drawbuffers);
+		else if (ncolortargets == 0 && hasDS && (GLAD_ES_VERSION_3_0 || !GLAD_ES_VERSION_2_0))
+		{
+			// glDrawBuffers is an ext in GL2. glDrawBuffer doesn't exist in ES3.
+			GLenum none = GL_NONE;
+			if (GLAD_ES_VERSION_3_0)
+				glDrawBuffers(1, &none);
+			else
+				glDrawBuffer(GL_NONE);
+			glReadBuffer(GL_NONE);
+		}
 
 		GLenum status = glCheckFramebufferStatus(GL_FRAMEBUFFER);
 
@@ -1397,9 +1410,9 @@ bool Graphics::isCanvasFormatSupported(PixelFormat format, bool readable) const
 	return Canvas::isFormatSupported(format, readable);
 }
 
-bool Graphics::isImageFormatSupported(PixelFormat format) const
+bool Graphics::isImageFormatSupported(PixelFormat format, bool sRGB) const
 {
-	return Image::isFormatSupported(format);
+	return Image::isFormatSupported(format, sRGB);
 }
 
 Shader::Language Graphics::getShaderLanguageTarget() const
