@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -45,6 +45,11 @@ static const char graphics_lua[] =
 #include "wrap_Graphics.lua"
 ;
 
+// This is in a separate file because VS2013 has a 16KB limit for raw strings..
+static const char graphics_shader_lua[] =
+#include "wrap_GraphicsShader.lua"
+;
+
 namespace love
 {
 namespace graphics
@@ -290,7 +295,7 @@ int w_setCanvas(lua_State *L)
 				targets.colors.emplace_back(luax_checkcanvas(L, -1), 0);
 
 				if (targets.colors.back().canvas->getTextureType() != TEXTURE_2D)
-					return luaL_error(L, "The table-of-tables variant of setCanvas must be used with non-2D Canvases.");
+					return luaL_error(L, "Non-2D canvases must use the table-of-tables variant of setCanvas.");
 			}
 
 			lua_pop(L, 1);
@@ -1091,7 +1096,7 @@ int w_newFont(lua_State *L)
 		for (int i = 0; i < lua_gettop(L); i++)
 			idxs.push_back(i + 1);
 
-		luax_convobj(L, &idxs[0], (int) idxs.size(), "font", "newRasterizer");
+		luax_convobj(L, idxs, "font", "newRasterizer");
 	}
 
 	love::font::Rasterizer *rasterizer = luax_checktype<love::font::Rasterizer>(L, 1);
@@ -1122,7 +1127,7 @@ int w_newImageFont(lua_State *L)
 		for (int i = 0; i < lua_gettop(L); i++)
 			idxs.push_back(i + 1);
 
-		luax_convobj(L, &idxs[0], (int) idxs.size(), "font", "newImageRasterizer");
+		luax_convobj(L, idxs, "font", "newImageRasterizer");
 	}
 
 	love::font::Rasterizer *rasterizer = luax_checktype<love::font::Rasterizer>(L, 1);
@@ -2245,7 +2250,7 @@ int w_getTextureTypes(lua_State *L)
 		if (!Texture::getConstant(textype, name))
 			continue;
 
-		lua_pushnumber(L, caps.textureTypes[i]);
+		luax_pushboolean(L, caps.textureTypes[i]);
 		lua_setfield(L, -2, name);
 	}
 
@@ -3075,6 +3080,11 @@ extern "C" int luaopen_love_graphics(lua_State *L)
 	else
 		lua_error(L);
 
+	if (luaL_loadbuffer(L, (const char *)graphics_shader_lua, sizeof(graphics_shader_lua), "wrap_GraphicsShader.lua") == 0)
+		lua_call(L, 0, 0);
+	else
+		lua_error(L);
+
 	return n;
 }
 
