@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -25,10 +25,10 @@ namespace love
 namespace image
 {
 
-ImageDataBase::ImageDataBase()
-	: format(PIXELFORMAT_UNKNOWN)
-	, width(0)
-	, height(0)
+ImageDataBase::ImageDataBase(PixelFormat format, int width, int height)
+	: format(format)
+	, width(width)
+	, height(height)
 {
 }
 
