@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -33,7 +33,6 @@ class ImageDataBase : public Data
 {
 public:
 
-	ImageDataBase();
 	virtual ~ImageDataBase() {}
 
 	PixelFormat getFormat() const;
@@ -45,6 +44,8 @@ public:
 
 protected:
 
+	ImageDataBase(PixelFormat format, int width, int height);
+
 	PixelFormat format;
 	int width;
 	int height;
