@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -23,9 +23,6 @@
 // LOVE
 #include "image/FormatHandler.h"
 
-// dds parser
-#include "ddsparse/ddsparse.h"
-
 // STL
 #include <string>
 
@@ -46,16 +43,13 @@ public:
 	virtual ~DDSHandler() {}
 
 	// Implements FormatHandler.
+	bool canDecode(Data *data) override;
+	DecodedImage decode(Data *data) override;
 	bool canParseCompressed(Data *data) override;
-
 	StrongRef<CompressedMemory> parseCompressed(Data *filedata,
 	        std::vector<StrongRef<CompressedSlice>> &images,
 	        PixelFormat &format, bool &sRGB) override;
 
-private:
-
-	static PixelFormat convertFormat(dds::Format ddsformat, bool &sRGB);
-
 }; // DDSHandler
 
 } // magpie
