@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -356,7 +356,7 @@ void JoystickModule::checkGamepads(const std::string &guid) const
 
 		for (auto stick : activeSticks)
 		{
-			if (stick->isGamepad() || guid.compare(stick->getGUID()) != 0)
+			if (guid.compare(stick->getGUID()) != 0)
 				continue;
 
 			// Big hack time: open the index as a game controller and compare
@@ -365,12 +365,15 @@ void JoystickModule::checkGamepads(const std::string &guid) const
 			if (controller == nullptr)
 				continue;
 
+			// GameController objects are reference-counted in SDL, so we don't want to
+			// have a joystick open when trying to re-initialize it
 			SDL_Joystick *sdlstick = SDL_GameControllerGetJoystick(controller);
-			if (sdlstick == (SDL_Joystick *) stick->getHandle())
-				stick->openGamepad(d_index);
-
-			// GameController objects are reference-counted in SDL.
+			bool open_gamepad = (sdlstick == (SDL_Joystick *) stick->getHandle());
 			SDL_GameControllerClose(controller);
+
+			// open as gamepad if necessary
+			if (open_gamepad)
+				stick->openGamepad(d_index);
 		}
 	}
 }
@@ -448,6 +451,25 @@ void JoystickModule::loadGamepadMappings(const std::string &mappings)
 		throw love::Exception("Invalid gamepad mappings.");
 }
 
+std::string JoystickModule::getGamepadMappingString(const std::string &guid) const
+{
+	SDL_JoystickGUID sdlguid = SDL_JoystickGetGUIDFromString(guid.c_str());
+
+	char *sdlmapping = SDL_GameControllerMappingForGUID(sdlguid);
+	if (sdlmapping == nullptr)
+		return "";
+
+	std::string mapping(sdlmapping);
+	SDL_free(sdlmapping);
+
+	// Matches SDL_GameControllerAddMappingsFromRW.
+	if (mapping.find_last_of(',') != mapping.length() - 1)
+		mapping += ",";
+	mapping += "platform:" + std::string(SDL_GetPlatform());
+
+	return mapping;
+}
+
 std::string JoystickModule::saveGamepadMappings()
 {
 	std::string mappings;
