@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -160,6 +160,17 @@ int w_saveGamepadMappings(lua_State *L)
 	return 1;
 }
 
+int w_getGamepadMappingString(lua_State *L)
+{
+	const char *guid = luaL_checkstring(L, 1);
+	std::string mapping = instance()->getGamepadMappingString(guid);
+	if (mapping.empty())
+		lua_pushnil(L);
+	else
+		luax_pushstring(L, mapping);
+	return 1;
+}
+
 // List of functions to wrap.
 static const luaL_Reg functions[] =
 {
@@ -168,6 +179,7 @@ static const luaL_Reg functions[] =
 	{ "setGamepadMapping", w_setGamepadMapping },
 	{ "loadGamepadMappings", w_loadGamepadMappings },
 	{ "saveGamepadMappings", w_saveGamepadMappings },
+	{ "getGamepadMappingString", w_getGamepadMappingString },
 	{ 0, 0 }
 };
 
