@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -69,6 +69,11 @@ extern "C"
 #	include "graphics/Graphics.h"
 #endif
 
+// For love::audio::Audio::setMixWithSystem.
+#ifdef LOVE_ENABLE_AUDIO
+#	include "audio/Audio.h"
+#endif
+
 // Scripts
 #include "scripts/nogame.lua.h"
 #include "scripts/boot.lua.h"
@@ -269,12 +274,18 @@ static int w_love_isVersionCompatible(lua_State *L)
 	std::string version;
 
 	if (lua_type(L, 1) == LUA_TSTRING)
+	{
 		version = luaL_checkstring(L, 1);
+
+		// Convert major.minor to major.minor.revision.
+		if (std::count(version.begin(), version.end(), '.') < 2)
+			version.append(".0");
+	}
 	else
 	{
 		int major = (int) luaL_checkinteger(L, 1);
 		int minor = (int) luaL_checkinteger(L, 2);
-		int rev   = (int) luaL_checkinteger(L, 3);
+		int rev   = (int) luaL_optinteger(L, 3, 0);
 
 		// Convert the numbers to a string, since VERSION_COMPATIBILITY is an
 		// array of version strings.
@@ -286,7 +297,13 @@ static int w_love_isVersionCompatible(lua_State *L)
 
 	for (int i = 0; love::VERSION_COMPATIBILITY[i] != nullptr; i++)
 	{
-		if (version.compare(love::VERSION_COMPATIBILITY[i]) != 0)
+		std::string v(love::VERSION_COMPATIBILITY[i]);
+
+		// Convert major.minor to major.minor.revision.
+		if (std::count(v.begin(), v.end(), '.') < 2)
+			v.append(".0");
+
+		if (version.compare(v) != 0)
 			continue;
 
 		lua_pushboolean(L, true);
@@ -305,6 +322,27 @@ static int w__setGammaCorrect(lua_State *L)
 	return 0;
 }
 
+static int w__setAudioMixWithSystem(lua_State *L)
+{
+	bool success = false;
+
+#ifdef LOVE_ENABLE_AUDIO
+	bool mix = love::luax_checkboolean(L, 1);
+	success = love::audio::Audio::setMixWithSystem(mix);
+#endif
+
+	love::luax_pushboolean(L, success);
+	return 1;
+}
+
+static int w__requestRecordingPermission(lua_State *L)
+{
+#ifdef LOVE_ENABLE_AUDIO
+	love::audio::setRequestRecordingPermission((bool) lua_toboolean(L, 1));
+#endif
+	return 0;
+}
+
 static int w_love_setDeprecationOutput(lua_State *L)
 {
 	bool enable = love::luax_checkboolean(L, 1);
@@ -362,6 +400,13 @@ int luaopen_love(lua_State *L)
 	lua_pushcfunction(L, w__setGammaCorrect);
 	lua_setfield(L, -2, "_setGammaCorrect");
 
+	// Exposed here because we need to be able to call it before the audio
+	// module is initialized.
+	lua_pushcfunction(L, w__setAudioMixWithSystem);
+	lua_setfield(L, -2, "_setAudioMixWithSystem");
+	lua_pushcfunction(L, w__requestRecordingPermission);
+	lua_setfield(L, -2, "_requestRecordingPermission");
+
 	lua_newtable(L);
 
 	for (int i = 0; love::VERSION_COMPATIBILITY[i] != nullptr; i++)
