@@ -3,7 +3,7 @@ R"luastring"--(
 -- There is a matching delimiter at the bottom of the file.
 
 --[[
-Copyright (c) 2006-2018 LOVE Development Team
+Copyright (c) 2006-2019 LOVE Development Team
 
 This software is provided 'as-is', without any express or implied
 warranty.  In no event will the authors be held liable for any damages
@@ -22,7 +22,7 @@ misrepresented as being the original software.
 3. This notice may not be removed or altered from any source distribution.
 --]]
 
-local love_math, ffifuncspointer = ...
+local love_math, ffifuncspointer_str = ...
 
 local type, tonumber, error = type, tonumber, error
 local floor = math.floor
@@ -58,6 +58,28 @@ function love_math.getRandomState()
 	return rng:getState()
 end
 
+function love_math.colorToBytes(r, g, b, a)
+	if type(r) == "table" then
+		r, g, b, a = r[1], r[2], r[3], r[4]
+	end
+	r = floor(clamp01(r) * 255 + 0.5)
+	g = floor(clamp01(g) * 255 + 0.5)
+	b = floor(clamp01(b) * 255 + 0.5)
+	a = a ~= nil and floor(clamp01(a) * 255 + 0.5) or nil
+	return r, g, b, a
+end
+
+function love_math.colorFromBytes(r, g, b, a)
+	if type(r) == "table" then
+		r, g, b, a = r[1], r[2], r[3], r[4]
+	end
+	r = clamp01(floor(r + 0.5) / 255)
+	g = clamp01(floor(g + 0.5) / 255)
+	b = clamp01(floor(b + 0.5) / 255)
+	a = a ~= nil and clamp01(floor(a + 0.5) / 255) or nil
+	return r, g, b, a
+end
+
 if type(jit) ~= "table" or not jit.status() then
 	-- LuaJIT's FFI is *much* slower than LOVE's regular methods when the JIT
 	-- compiler is disabled.
@@ -81,7 +103,7 @@ typedef struct FFI_Math
 } FFI_Math;
 ]])
 
-local ffifuncs = ffi.cast("FFI_Math *", ffifuncspointer)
+local ffifuncs = ffi.cast("FFI_Math **", ffifuncspointer_str)[0]
 
 
 -- Overwrite some regular love.math functions with FFI implementations.
