@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2016 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -21,13 +21,11 @@
 #ifndef LOVE_SOUND_LULLABY_FLAC_DECODER_H
 #define LOVE_SOUND_LULLABY_FLAC_DECODER_H
 
-#if 0
-
 // LOVE
 #include "common/Data.h"
-#include "Decoder.h"
+#include "sound/Decoder.h"
 
-#include <FLAC++/decoder.h>
+#include "dr_flac/dr_flac.h"
 #include <string.h>
 
 namespace love
@@ -37,41 +35,29 @@ namespace sound
 namespace lullaby
 {
 
-class FLACDecoder : public Decoder, public FLAC::Decoder::Stream
+class FLACDecoder : public Decoder
 {
 public:
-	FLACDecoder(Data *data, const std::string &ext, int bufferSize);
+	FLACDecoder(Data *data, int bufferSize);
 	~FLACDecoder();
 
 	static bool accepts(const std::string &ext);
 	love::sound::Decoder *clone();
 	int decode();
-	bool seek(float s);
+	bool seek(double s);
 	bool rewind();
 	bool isSeekable();
-	int getChannels() const;
+	int getChannelCount() const;
 	int getBitDepth() const;
 	int getSampleRate() const;
 	double getDuration();
 
-	//needed for FLAC
-	FLAC__StreamDecoderReadStatus read_callback(FLAC__byte buffer[], size_t *bytes);
-	FLAC__StreamDecoderSeekStatus seek_callback(FLAC__uint64 offset);
-	FLAC__StreamDecoderTellStatus tell_callback(FLAC__uint64 *offset);
-	FLAC__StreamDecoderLengthStatus length_callback(FLAC__uint64 *length);
-	bool eof_callback();
-	FLAC__StreamDecoderWriteStatus write_callback(const FLAC__Frame *frame, const FLAC__int32 *const buffer[]);
-	void metadata_callback(const FLAC__StreamMetadata *metadata);
-	void error_callback(FLAC__StreamDecoderErrorStatus status);
-
 private:
-	int pos;
+	drflac *flac;
 }; // Decoder
 
 } // lullaby
 } // sound
 } // love
 
-#endif // 0
-
 #endif // LOVE_SOUND_LULLABY_FLAC_DECODER_H
