@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -128,8 +128,8 @@ static long vorbisTell(void *datasource	/* ptr to the data that the vorbis files
  * END CALLBACK FUNCTIONS
  **/
 
-VorbisDecoder::VorbisDecoder(Data *data, const std::string &ext, int bufferSize)
-	: Decoder(data, ext, bufferSize)
+VorbisDecoder::VorbisDecoder(Data *data, int bufferSize)
+	: Decoder(data, bufferSize)
 	, duration(-2.0)
 {
 	// Initialize callbacks
@@ -182,7 +182,7 @@ bool VorbisDecoder::accepts(const std::string &ext)
 
 love::sound::Decoder *VorbisDecoder::clone()
 {
-	return new VorbisDecoder(data.get(), ext, bufferSize);
+	return new VorbisDecoder(data.get(), bufferSize);
 }
 
 int VorbisDecoder::decode()
@@ -209,7 +209,7 @@ int VorbisDecoder::decode()
 	return size;
 }
 
-bool VorbisDecoder::seek(float s)
+bool VorbisDecoder::seek(double s)
 {
 	int result = 0;
 
