@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -23,6 +23,7 @@
 
 // STL
 #include <string>
+#include <map>
 
 // LOVE
 #include "common/Data.h"
@@ -51,6 +52,11 @@ public:
 	virtual const char *getName() const;
 	virtual ModuleType getModuleType() const { return M_THREAD; }
 
+private:
+
+	std::map<std::string, StrongRef<Channel>> namedChannels;
+	MutexRef namedChannelMutex;
+
 }; // ThreadModule
 
 } // thread
