@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2018 LOVE Development Team
+ * Copyright (c) 2006-2019 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -90,7 +90,7 @@ double Timer::step()
 
 void Timer::sleep(double seconds) const
 {
-	if (seconds > 0)
+	if (seconds >= 0)
 		love::sleep((unsigned int)(seconds*1000));
 }
 
