@@ -1,5 +1,5 @@
 --[[
-Copyright (c) 2006-2018 LOVE Development Team
+Copyright (c) 2006-2019 LOVE Development Team
 
 This software is provided 'as-is', without any express or implied
 warranty.  In no event will the authors be held liable for any damages
@@ -272,6 +272,9 @@ function love.createhandlers()
 			collectgarbage()
 			collectgarbage()
 		end,
+		displayrotated = function (display, orient)
+			if love.displayrotated then return love.displayrotated(display, orient) end
+		end,
 	}, {
 		__index = function(self, name)
 			error("Unknown event: " .. name)
@@ -398,6 +401,7 @@ function love.init()
 			resizable = false,
 			centered = true,
 			highdpi = false,
+			usedpiscale = true,
 		},
 		modules = {
 			data = true,
@@ -421,6 +425,7 @@ function love.init()
 		},
 		audio = {
 			mixwithsystem = true, -- Only relevant for Android / iOS.
+			mic = false, -- Only relevant for Android.
 		},
 		console = false, -- Only relevant for windows.
 		identity = false,
@@ -465,6 +470,16 @@ function love.init()
 		love._setGammaCorrect(c.gammacorrect)
 	end
 
+	if love._setAudioMixWithSystem then
+		if c.audio and c.audio.mixwithsystem ~= nil then
+			love._setAudioMixWithSystem(c.audio.mixwithsystem)
+		end
+	end
+
+	if love._requestRecordingPermission then
+		love._requestRecordingPermission(c.audio and c.audio.mic)
+	end
+
 	-- Gets desired modules.
 	for k,v in ipairs{
 		"data",
@@ -532,6 +547,7 @@ function love.init()
 			centered = c.window.centered,
 			display = c.window.display,
 			highdpi = c.window.highdpi,
+			usedpiscale = c.window.usedpiscale,
 			x = c.window.x,
 			y = c.window.y,
 		}), "Could not set window mode")
@@ -542,10 +558,6 @@ function love.init()
 		end
 	end
 
-	if love.audio then
-		love.audio.setMixWithSystem(c.audio.mixwithsystem)
-	end
-
 	-- Our first timestep, because window creation can take some time
 	if love.timer then
 		love.timer.step()
